﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Bluetooth.Widcomm
{
	public delegate void InquiryEvent(object sender, InquiryEventArgs e);

	public class WidcommStack : System.IDisposable
	{
		private IntPtr			m_pStack	= IntPtr.Zero;
		private bool			m_bDisposed = false;
		private StackNotifySink m_sink		= new StackNotifySink();

		public WidcommStack()
		{
			m_pStack		= WidcommAPI.CreateStack();
			m_sink.Stack	= this;
		}

		~WidcommStack()
		{
			Dispose(false);
		}

		public event InquiryEvent InquiryStateChanged
		{
			add { m_sink.InquiryStateChanged += value; }
			remove { m_sink.InquiryStateChanged -= value; }
		}

		public bool StartInquiry()
		{
			WidcommAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_INQUIRY);

			return WidcommAPI.StackStartInquiry(m_pStack) != 0;
		}

		public void StopInquiry()
		{
			WidcommAPI.StackStopInquiry(m_pStack);
		}

		/// <summary>
		/// Gets the version of the Widcomm stack.
		/// </summary>
		/// <remarks>
		/// Returns zero for no stack present.
		/// </remarks>
		public static int Version
		{
			get { return WidcommAPI.WidcommStackPresent(); }
		}

		protected void Dispose(bool bDisposing)
		{
			if(m_bDisposed)
				return;

			if(m_pStack != IntPtr.Zero)
				WidcommAPI.DeleteStack(m_pStack);
			m_pStack = IntPtr.Zero;

			m_bDisposed = true;
		}


		#region IDisposable Members

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
