﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Primeworks.Bluetooth;

namespace Enumerate
{
	public partial class MainForm : Form
	{
		private bool						m_bEnumerating	= false;
		private BluetoothClient				m_client		= new BluetoothClient();
		private BluetoothDevice				m_device		= null;
		private ServiceDiscoveryEndedEvent	m_serviceEvent;

		public MainForm()
		{
			InitializeComponent();

			m_client.DeviceFound += new DeviceFoundEventHandler(m_client_DeviceFound);

			m_serviceEvent = new ServiceDiscoveryEndedEvent(device_ServiceDiscoveryEnded);
		}

		void m_client_DeviceFound(object sender, DeviceDiscoveryCancelEventArgs e)
		{
			if(e.NoMoreDevices)
			{
				MessageBox.Show("Finished device inquiry", "Status");
				menuItemEnumDevices.Text = "Enumerate Devices";
				menuItemExit.Enabled = true;
				m_bEnumerating = false;
				menuListServices.Enabled = true;
			}
			else
			{
				ListViewItem item = new ListViewItem(e.Device.Name);
				StringBuilder str = new StringBuilder();
				byte[] addr = e.Device.AddressAsBytes;

				str.Append(addr[0].ToString("X2"));
				str.Append(":");
				str.Append(addr[1].ToString("X2"));
				str.Append(":");
				str.Append(addr[2].ToString("X2"));
				str.Append(":");
				str.Append(addr[3].ToString("X2"));
				str.Append(":");
				str.Append(addr[4].ToString("X2"));
				str.Append(":");
				str.Append(addr[5].ToString("X2"));

				item.SubItems.Add(e.Device.Class.ToString("X"));
				item.SubItems.Add(str.ToString());

				item.Tag = e.Device;

				listDevices.Items.Add(item);
			}
		}

		private void menuItemExit_Click(object sender, EventArgs e)
		{
			Close();
		}

		private void menuItemEnumDevices_Click(object sender, EventArgs e)
		{
			if(!m_bEnumerating)
			{
				if(m_client.BeginDeviceDiscovery())
				{
					m_bEnumerating = true;

					listDevices.Items.Clear();
					menuItemEnumDevices.Text = "Stop";
					menuItemExit.Enabled = false;
				}
				else
					MessageBox.Show("Failed to start device discovery.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
			}
		}

		private void MainForm_Closing(object sender, CancelEventArgs e)
		{
			m_client.Dispose();
		}

		private void listMenu_Popup(object sender, EventArgs e)
		{
		}

		private void menuListServices_Click(object sender, EventArgs e)
		{
			if(listDevices.SelectedIndices.Count == 1)
			{
				int				iItem	= listDevices.SelectedIndices[0];
				ListViewItem	item	= listDevices.Items[iItem];
				
				m_device = item.Tag as BluetoothDevice;

				if(m_device.BeginServiceDiscovery())
				{
					m_device.ServiceDiscoveryEnded += m_serviceEvent;
					menuListServices.Enabled = false;
				}
				else
				{
					MessageBox.Show("Failed to start service discovery.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
				}
			}
		}

		void device_ServiceDiscoveryEnded(object sender, ServiceDiscoveryEventArgs e)
		{
			ServicesForm servicesDlg = new ServicesForm(m_device);

			servicesDlg.ShowDialog();

			menuListServices.Enabled = true;
			m_device.ServiceDiscoveryEnded -= m_serviceEvent;
		}
	}
}