﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Primeworks.Bluetooth
{
	public delegate void InquiryEvent(object sender, InquiryEventArgs e);
	public delegate void ServiceDiscoveryEndedEvent(object sender, ServiceDiscoveryEventArgs e);

	public class BluetoothStack
	{
		protected IntPtr			m_pStack	= IntPtr.Zero;
		protected StackNotifySink	m_sink		= new StackNotifySink();

		public BluetoothStack()
		{
		}

		public virtual bool StartInquiry()
		{
			return false;
		}

		public virtual void StopInquiry()
		{
		}

		/// <summary>
		/// Begins the service discovery process for the given device.
		/// </summary>
		/// <param name="btAddress">
		/// Bluetooth address of device.
		/// </param>
		/// <returns>
		/// <b>true</b> if the discovery process started successfully.
		/// </returns>
		public virtual bool BeginServiceDiscovery(long btAddress)
		{
			return false;
		}

		/// <summary>
		/// Begins the service discovery process for the given device.
		/// </summary>
		/// <param name="btAddress">
		/// Bluetooth address of device.
		/// </param>
		/// <param name="serviceGuid">
		/// Specific service GUID to search for.
		/// </param>
		/// <returns>
		/// <b>true</b> if the discovery process started successfully.
		/// </returns>
		public virtual bool BeginServiceDiscovery(long btAddress, Guid serviceGuid)
		{
			return false;
		}

		/// <summary>
		/// Gets a ServiceDiscovery object containing all the services found for a device
		/// </summary>
		/// <param name="btAddress">
		/// Bluetooth address of device.
		/// </param>
		/// <returns>
		/// A <see cref="ServiceDiscovery"/>-derived object.
		/// </returns>
		public virtual ServiceDiscovery GetServiceDiscovery(long btAddr)
		{
			throw new NotImplementedException();
		}

		public event InquiryEvent InquiryStateChanged
		{
			add { m_sink.InquiryStateChanged += value; }
			remove { m_sink.InquiryStateChanged -= value; }
		}

		public event ServiceDiscoveryEndedEvent ServiceDiscoveryEnded
		{
			add { m_sink.ServiceDiscoveryEnded += value; }
			remove { m_sink.ServiceDiscoveryEnded -= value; }
		}

		public virtual void Dispose()
		{
		}
	}
}
