﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Primeworks.Bluetooth.Microsoft
{
	public class MicrosoftServiceDiscovery : ServiceDiscovery, System.IDisposable
	{
		private IntPtr m_pDiscovery = IntPtr.Zero;

		internal MicrosoftServiceDiscovery(IntPtr pDiscovery)
		{
			m_pDiscovery = pDiscovery;
		}

		public override BluetoothServiceList Services
		{
			get
			{
				BluetoothServiceList	services	= new BluetoothServiceList();
				int						i;
				int						nServices	= MicrosoftAPI.DiscoveryGetSdpRecCount(m_pDiscovery);


				for(i = 0; i < nServices; ++i)
				{
					IntPtr			pSdpRec = MicrosoftAPI.DiscoveryGetSdpRec(m_pDiscovery, i);
					StringBuilder	strName = new StringBuilder(256);
					Guid			guid	= new Guid();
					int				nRFComm = -1;

					if(MicrosoftAPI.SdpRecFindServiceName(pSdpRec, strName, 255) == 0)
						strName.Append("No name found");

					MicrosoftAPI.SdpRecFindServiceClass(pSdpRec, out guid);

					if(MicrosoftAPI.SdpRecFindRFCommChannel(pSdpRec, ref nRFComm) == 0)
						nRFComm = -1;

					services.Add(new BluetoothService(strName.ToString(), guid, nRFComm));
				}
				return services;
			}
		}

		#region IDisposable Members

		/// <summary>
		/// The Microsoft version of this class uses static data, so Dispose is not needed.
		/// </summary>
		public void Dispose()
		{
		}

		#endregion
	}
}
