﻿using System;
using System.Collections.Generic;
using System.Text;

using Primeworks.Bluetooth;

namespace Primeworks.Bluetooth.Microsoft
{
	public class MicrosoftStack : Primeworks.Bluetooth.BluetoothStack, System.IDisposable
	{
		private bool m_bDisposed = false;

		public MicrosoftStack()
		{
			m_pStack = MicrosoftAPI.CreateStack();
			m_sink.Stack = this;
		}

		~MicrosoftStack()
		{
			Dispose(false);
		}

		public override bool StartInquiry()
		{
			MicrosoftAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_INQUIRY);

			return MicrosoftAPI.StackStartInquiry(m_pStack) != 0;
		}

		public override void StopInquiry()
		{
			MicrosoftAPI.StackStopInquiry(m_pStack);
		}

		public override bool BeginServiceDiscovery(long btAddress)
		{
			Guid guid = BluetoothService.L2CapProtocol;

			MicrosoftAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_DISCOVERY);

			return MicrosoftAPI.StackStartDiscovery(m_pStack, btAddress, ref guid) != 0;
		}

		public override bool BeginServiceDiscovery(long btAddress, Guid serviceGuid)
		{
			MicrosoftAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_DISCOVERY);

			return MicrosoftAPI.StackStartDiscovery(m_pStack, btAddress, ref serviceGuid) != 0;
		}

		public override ServiceDiscovery GetServiceDiscovery(long btAddr)
		{
			IntPtr						pDiscovery	= MicrosoftAPI.StackGetDiscovery(m_pStack, btAddr, IntPtr.Zero);
			MicrosoftServiceDiscovery	discovery	= new MicrosoftServiceDiscovery(pDiscovery);

			return discovery;
		}

		protected void Dispose(bool bDisposing)
		{
			if(m_bDisposed)
				return;

			if(m_pStack != IntPtr.Zero)
				MicrosoftAPI.DeleteStack(m_pStack);
			m_pStack = IntPtr.Zero;

			m_bDisposed = true;
		}


		#region IDisposable Members

		public override void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
