﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using Microsoft.WindowsCE.Forms;

namespace Primeworks.Bluetooth
{
	public class StackNotifySink : MessageWindow
	{
		public const int WM_NOTIFY_INQUIRY		= 0x400 + 0x100;
		public const int WM_NOTIFY_DISCOVERY	= 0x400 + 0x101;

		private BluetoothStack		m_stack				= null;
		private InquiryEvent		m_eventInqury		= null;
		private ServiceDiscoveryEndedEvent m_eventDiscovery	= null;

		public StackNotifySink()
		{
		}

		public BluetoothStack Stack
		{
			get { return m_stack; }
			set { m_stack = value; }
		}

		public event InquiryEvent InquiryStateChanged
		{
			add { m_eventInqury += value; }
			remove { m_eventInqury -= value; }
		}

		public event ServiceDiscoveryEndedEvent ServiceDiscoveryEnded
		{
			add { m_eventDiscovery += value; }
			remove { m_eventDiscovery -= value; }
		}

		private void OnInquiryStateChanged(int nState, IntPtr pDevInfo)
		{
			if(m_eventInqury != null && m_stack != null)
			{
				InquiryEventArgs e;

				if(nState == 0)
				{
					BluetoothDevice devInfo = new BluetoothDevice(m_stack, pDevInfo);

					e = new InquiryEventArgs(devInfo);

					Marshal.FreeHGlobal(pDevInfo);
				}
				else
					e = new InquiryEventArgs();

				m_eventInqury(m_stack, e);
			}
		}

		private void OnDiscoveryEnded(int nError, IntPtr pRecords)
		{
			if(m_eventDiscovery != null)
			{
				ServiceDiscoveryEventArgs e = new ServiceDiscoveryEventArgs(nError, pRecords);

				m_eventDiscovery(m_stack, e);
			}
		}

		protected override void WndProc(ref Message msg)
		{
			if(msg.Msg == WM_NOTIFY_INQUIRY)
			{
				int		wParam = msg.WParam.ToInt32();
				IntPtr	lParam = (IntPtr)msg.LParam.ToInt32();

				OnInquiryStateChanged(wParam, lParam);
			}
			else if(msg.Msg == WM_NOTIFY_DISCOVERY)
			{
				OnDiscoveryEnded(msg.WParam.ToInt32(), (IntPtr)msg.LParam.ToInt32());
			}

			base.WndProc(ref msg);
		}
	}
}
