﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Primeworks.Bluetooth.Widcomm
{
	public class WidcommServiceDiscovery : ServiceDiscovery, System.IDisposable
	{
		private IntPtr	m_pDiscovery	= IntPtr.Zero;
		private bool	m_bDisposed		= false;

		internal WidcommServiceDiscovery(IntPtr pDiscovery)
		{
			m_pDiscovery = pDiscovery;
		}

		~WidcommServiceDiscovery()
		{
			Dispose(false);
		}

		public override BluetoothServiceList Services
		{
			get
			{
				BluetoothServiceList	services	= new BluetoothServiceList();
				int						i;
				int						nServices	= WidcommAPI.DiscoveryGetSdpRecCount(m_pDiscovery);

				for(i = 0; i < nServices; ++i)
				{
					IntPtr			pSdpRec = WidcommAPI.DiscoveryGetSdpRec(m_pDiscovery, i);
					StringBuilder	strName	= new StringBuilder(256);
					Guid			guid	= new Guid();
					int				nRFComm = -1;

					if(WidcommAPI.SdpRecFindServiceName(pSdpRec, strName, 255) == 0)
						strName.Append("No name found");

					WidcommAPI.SdpRecFindServiceClass(pSdpRec, out guid);

					if(WidcommAPI.SdpRecFindRFCommChannel(pSdpRec, ref nRFComm) == 0)
						nRFComm = -1;

					services.Add(new BluetoothService(strName.ToString(), guid, nRFComm));
				}

				return services;
			}
		}

		protected void Dispose(bool bDisposing)
		{
			if(m_bDisposed)
				return;

			if(m_pDiscovery != IntPtr.Zero)
				WidcommAPI.DeleteDiscovery(m_pDiscovery);

			m_bDisposed = true;
		}

		#region IDisposable Members

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
