﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using Primeworks.Bluetooth;

namespace Primeworks.Bluetooth.Widcomm
{
	public class WidcommStack : Primeworks.Bluetooth.BluetoothStack, System.IDisposable
	{
		private bool	m_bDisposed = false;
		private long	m_btAddress	= 0;

		public WidcommStack()
		{
			m_pStack = WidcommAPI.CreateStack();
			m_sink.Stack = this;
		}

		~WidcommStack()
		{
			Dispose(false);
		}

		public override bool StartInquiry()
		{
			WidcommAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_INQUIRY);

			return WidcommAPI.StackStartInquiry(m_pStack) != 0;
		}

		public override void StopInquiry()
		{
			WidcommAPI.StackStopInquiry(m_pStack);
		}

		public override bool BeginServiceDiscovery(long btAddress)
		{
			m_btAddress = btAddress;

			WidcommAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_DISCOVERY);

			return WidcommAPI.StackStartDiscoveryPtr(m_pStack, btAddress, IntPtr.Zero) != 0;
		}

		public override bool BeginServiceDiscovery(long btAddress, Guid serviceGuid)
		{
			m_btAddress = btAddress;

			WidcommAPI.StackNotifyWindow(m_pStack, m_sink.Hwnd, StackNotifySink.WM_NOTIFY_DISCOVERY);

			return WidcommAPI.StackStartDiscovery(m_pStack, btAddress, ref serviceGuid) != 0;
		}

		/// <summary>
		/// Returns a new <see cref="WidcommServiceDiscovery"/> object containing all the 
		/// <see cref="SdpRecod"/> information for the given device.
		/// </summary>
		/// <param name="btAddr">
		/// Bluetooth device address.
		/// </param>
		/// <returns>
		/// A <see cref="WidcommServiceDiscovery"/> object.
		/// </returns>
		public override ServiceDiscovery GetServiceDiscovery(long btAddr)
		{
			IntPtr					pDiscovery	= WidcommAPI.StackGetDiscovery(m_pStack, btAddr, IntPtr.Zero);
			WidcommServiceDiscovery discovery	= new WidcommServiceDiscovery(pDiscovery);

			return discovery;
		}

		/// <summary>
		/// Gets the version of the Widcomm stack.
		/// </summary>
		/// <remarks>
		/// Returns zero for no stack present.
		/// </remarks>
		public static int Version
		{
			get
			{
				try
				{
					return WidcommAPI.WidcommStackPresent();
				}
				catch
				{
					return 0;
				}
			}
		}

		protected void Dispose(bool bDisposing)
		{
			if(m_bDisposed)
				return;

			if(m_pStack != IntPtr.Zero)
				WidcommAPI.DeleteStack(m_pStack);
			m_pStack = IntPtr.Zero;

			m_bDisposed = true;
		}


		#region IDisposable Members

		public override void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
